      **************************************************************************
      *
      * Copyright (C) Micro Focus 2015-2016.
      * All rights reserved.
      *
      * This sample code is supplied for demonstration purposes only
      * on an "as is" basis and is for use at your own risk.
      *
      **************************************************************************
       class-id WindowsLeases.PropertyView is partial
                 inherits type System.Windows.Forms.Form.
       working-storage section. 
       01 rentalProperty           type RentalProperty.           
       01 store                type AgencyStore.
       method-id NEW (store as type AgencyStore).
           set self::store to store
           invoke self::InitializeComponent()
           invoke self::addressFields::EnableEditing(true)
           set self::okBtn::Enabled to true
       end method. 
     
       method-id NEW (store as type AgencyStore, rp as type RentalProperty).
           set self::store to store
           invoke self::InitializeComponent()
           invoke self::addressFields::EnableEditing(false)
           set rentalProperty to rp
           set self::addressFields::AddressLine1 = rp::Address::Address1
           set self::addressFields::City = rp::Address::City
           set self::addressFields::State = rp::Address::State
           set self::addressFields::PostalCode = rp::Address::PostalCode
           set self::rentBox::Text to rp::MonthlyRent
           attach method RentalPropertyUpdated to rentalProperty::changeEvent
       end method.

       method-id okBtn_Click final private.
       procedure division using by value sender as object e as type System.EventArgs.
           if rentalProperty = null
               declare rental = CreateRentalProperty()
               if rental <> null
                   invoke store::Save(rental)
               end-if
               invoke self::Close()
           else
               invoke UpdateRent()
               invoke self::Close()
           end-if
       end method.

       method-id UpdateRent.
           try
               declare rent as binary-long = type Int32::Parse(self::rentBox::Text) 
               if rent <> rentalProperty::MonthlyRent 
                   set rentalProperty::MonthlyRent to rent
                   invoke store::Save(rentalProperty)
               end-if
           catch ex as type FormatException
               invoke DisplayNonNumericRentMessage()
           end-try
       end method.

       method-id CreateRentalProperty returning result as type RentalProperty.
           if size of addressFields::AddressLine1 > 0 
                           and size of addressFields::PostalCode > 0
                   declare addressKey = new Address(addressFields::AddressLine1, 
                                                    addressFields::City,
                                                    addressFields::State,
                                                    addressFields::PostalCode)
                   declare existing = store::FindProperty(addressKey)
                   
                   if existing = null
                       declare rentText = self::rentBox::Text 
                       try
                            declare rent as binary-long = type Int32::Parse(rentText)
                            declare rental = new RentalProperty(addressKey)
                            set rental::MonthlyRent to rent
                            set result to rental                                        
                       catch e as type FormatException
                           invoke DisplayNonNumericRentMessage()
                       catch e as type PersistenceException
                            invoke type MessageBox::Show(self, 
                                type Messages::PERSISTENCE_ERROR & e::Message, 
                                type Messages::ADD_RENTAL_MSG_TITLE, 
                                type MessageBoxButtons::OK, 
                                type MessageBoxIcon::Error)
                        end-try
                   else
                      invoke type MessageBox::Show(self, 
                               type Messages::RENTAL_PROPERTY_EXISTS
                               type Messages::ADD_RENTAL_MSG_TITLE,
                               type MessageBoxButtons::OK, 
                               type MessageBoxIcon::Error)
                   end-if
           else
               invoke type MessageBox::Show(self, 
                               type Messages::ADDRESS_KEY_FIELDS_EMPTY, 
                               type Messages::ADD_RENTAL_MSG_TITLE, 
                               type MessageBoxButtons::OK, 
                               type MessageBoxIcon::Error)
           end-if
       end method. 
       
       method-id DisplayNonNumericRentMessage().
           invoke type MessageBox::Show(self, 
                type Messages::NUMERIC_RENTAL_VALUE, 
                type Messages::ADD_RENTAL_MSG_TITLE, 
                type MessageBoxButtons::OK, 
                type MessageBoxIcon::Error)
       end method. 

       method-id button2_Click final private.
       procedure division using by value sender as object e as type System.EventArgs.
           if rentalProperty <> null
               detach method RentalPropertyUpdated from rentalProperty::changeEvent
           end-if
           invoke self::Close()
       end method.

       method-id applyBtn_Click final private.
       procedure division using by value sender as object e as type System.EventArgs.
           invoke addressFields::EnableEditing(false)
           if rentalProperty = null
               set rentalProperty to CreateRentalProperty()
           else
               invoke UpdateRent()
           end-if
       end method.
       
       method-id RentalPropertyUpdated(o as type Updateable).
           if o instance of type RentalProperty *> this should always be true
               set rentBox::Text to (o as type RentalProperty)::MonthlyRent
           end-if
       end method. 

       end class.
