      **************************************************************************
      *
      * Copyright (C) Micro Focus 2015-2016.
      * All rights reserved.
      *
      * This sample code is supplied for demonstration purposes only
      * on an "as is" basis and is for use at your own risk.
      *
      **************************************************************************
       class-id MicroFocus.COBOL.Store.AgencyStore public.
       copy "fileoperations.cpy" replacing ==(prefix)== by ==opcode==. 
       
       01 rentalDictionary             dictionary[type Address, type RentalProperty].
       01 landlordDictionary           dictionary[string, type Landlord].
       01 tenantDictionary             dictionary[string, type Tenant].
       01 leaseDictionary              dictionary[string, type Lease].
       01 rentalFile                   type RentalPropertyFile static value new RentalPropertyFile().
       01 personFile                   type PersonFile static value new PersonFile(). 
       01 leaseFile                    type LeaseFile static value new LeaseFile(). 
       
       method-id New.
           create landlordDictionary
           create tenantDictionary
           create rentalDictionary
           create leaseDictionary
       end method. 
       
       method-id Save(rental as type RentalProperty).
           declare fs = new RentalPropertyFile+Status()
           declare rentalRecord = new RentalPropertyFile+Property()
           invoke RegisterUpdateable(rental)
           set rentalDictionary[rental::Address] to rental
           invoke AddressToPropertyRecord(rental::Address rentalRecord)          
           invoke rentalFile::RentalPropertyFile(read-record 
                                                 rentalRecord fs) 
           invoke PropertyToRecord(rental rentalRecord)                                          
           if fs::Status = statusOk
              invoke rentalFile::RentalPropertyFile(update-record 
                                                     rentalRecord fs) 
           else 
               if fs::Status = statusNotFound or fs::Status = statusFileNotFound
                                               or fs::Status = statusIndexFileNotFound
                   invoke rentalFile::RentalPropertyFile(add-record 
                                                     rentalRecord fs)
               end-if
           end-if
           if fs::Status <> statusOK 
               raise new PersistenceException("Exception saving record, status " & fs::Status)
           end-if  
       end method. 
       
       method-id Save(landlord as type Landlord).
            set landlordDictionary[landlord::Email::Address] to landlord 
           invoke Save(landlord as type Person)
       end method. 
       
       method-id Save(tenant  as type Tenant). 
            set tenantDictionary[tenant::Email::Address] to tenant
           invoke Save(tenant as type Person)
       end method. 
       
       method-id Save(person as type Person) protected.
           declare fs = new PersonFile+Status()
           declare personRecord = new PersonFile+Person()

           set personRecord::Email to person::Email
           invoke RegisterUpdateable(person)           
           invoke personFile::PersonFile(read-record 
                                                 personRecord fs) 
           invoke self::PersonToRecord(person personRecord)
           
          if fs::Status = statusOk 
              invoke personFile::PersonFile(update-record personRecord fs)
           else 
               if fs::Status = statusNotFound or fs::Status = statusFileNotFound
                                               or fs::Status = statusIndexFileNotFound
                   invoke personFile::PersonFile(add-record personRecord fs)
               end-if
           end-if
           if fs::Status <> statusOk 
               raise new PersistenceException("Exception saving record, status " & fs::Status)
           end-if  
       end method. 
       
       method-id Save(lease as type Lease).
          set leaseDictionary[lease::Id] to lease
          invoke RegisterUpdateable(lease)           


           declare fs = new LeaseFile+Status()
           declare leaseRecord = new LeaseFile+Lease()

           set leaseRecord::Id to lease::Id      
           invoke leaseFile::LeaseFile(read-record 
                                                 leaseRecord fs) 
           invoke self::LeaseToRecord(lease leaseRecord)
           
          if fs::Status = statusOk 
              invoke leaseFile::LeaseFile(update-record leaseRecord fs)
           else 
               if fs::Status = statusNotFound or fs::Status = statusFileNotFound
                                               or fs::Status = statusIndexFileNotFound
                   invoke leaseFile::LeaseFile(add-record leaseRecord fs)
               end-if
           end-if
           if fs::Status <> statusOk 
               raise new PersistenceException("Exception saving record, status " & fs::Status)
           end-if  
           
       end method. 
       
       method-id FindLease(leaseId as string) returning result as type Lease. 
           if leaseDictionary contains leaseId
               set result to leaseDictionary[leaseId]
           end-if
       end method. 
       
       method-id FindTenant(email as string) returning result as type Tenant. 
           if tenantDictionary contains email
               set result to tenantDictionary[email]
           end-if
       end method.  
       
       method-id FindLandlord(email as string) returning result as type Landlord. 
           if landlordDictionary contains email
               set result to landlordDictionary[email]
           end-if
       end method.  
       
       method-id FindProperty(rentalAddress as type Address) 
                    returning result as type RentalProperty. 
           if rentalDictionary contains rentalAddress
               set result to rentalDictionary[rentalAddress]
           end-if
       end method.
       
       iterator-id GetLandlords yielding nextLandlord as type Landlord.
          perform varying value landlord as type Landlord through landlordDictionary
               set nextLandlord to landlord
               goback 
           end-perform
       end iterator.
       
       iterator-id GetTenants yielding nextTenant as type Tenant.
           perform varying value tenant as type Tenant through tenantDictionary
               set nextTenant to tenant
               goback 
           end-perform
       end iterator.

       iterator-id GetLeases yielding nextLease as type Lease.
           perform varying value lease as type Lease through leaseDictionary
               set nextLease to lease
               goback 
           end-perform
       end iterator.
       
       iterator-id GetRentalProperties yielding nextProperty as type RentalProperty.
           perform varying value rental as type RentalProperty through rentalDictionary
               set nextProperty to rental
               goback 
           end-perform
       end iterator.


      
       
       method-id LoadCache(). 
           invoke LoadAllPersons()
           invoke LoadAllRentals()
           invoke LoadAllLeases()
       end method. 
    
       method-id LoadAllPersons() private.
           declare fs = new PersonFile+Status()
           declare nextKey = ""
           perform until exit
               declare personRecord = new PersonFile+Person()
               set personRecord::Email = nextKey
               invoke personFile::PersonFile(next-record personRecord fs)
               if fs::Status <> statusOk
                   exit perform 
               end-if
               set nextKey = personRecord::Email
               declare person = RecordToPerson(personRecord)
               invoke RegisterUpdateable(person)           
               if person instance of type Landlord
                   write landlordDictionary from person as type Landlord 
                                            key person::Email::Address
               else 
                   write tenantDictionary from person as type Tenant 
                                          key person::Email::Address
                end-if
           end-perform 
       end method.
       
           
       method-id LoadAllLeases() private. 
           declare fs = new LeaseFile+Status()
           declare nextKey = ""
           perform until exit
               declare leaseRecord = new LeaseFile+Lease()
               set leaseRecord::Id = nextKey
               invoke leaseFile::LeaseFile(next-record leaseRecord fs)
               if fs::Status <> statusOk
                   exit perform 
               end-if
               set nextKey = leaseRecord::Id
               declare lease = self::RecordToLease(leaseRecord)
               invoke RegisterUpdateable(lease)
               set leaseDictionary[lease::Id] to lease        
           end-perform 
       end method.
       
       method-id LoadAllRentals() private.
           declare fs = new RentalPropertyFile+Status()
           declare nextKey = ""
           perform until exit
               declare rentalRecord = new RentalPropertyFile+Property()
               set rentalRecord::AddressKey = nextKey
               invoke rentalFile::RentalPropertyFile(next-record rentalRecord fs)
               if fs::Status <> statusOk
                   exit perform 
               end-if
               set nextKey = rentalRecord::AddressKey
               declare rentalProperty = RecordToProperty(rentalRecord)
               invoke RegisterUpdateable(rentalProperty)
               write rentalDictionary from rentalProperty key rentalProperty::Address
            end-perform 
       end method.
       
       method-id RegisterUpdateable(o as type Updateable).
           detach method UpdateRecord from o::changeEvent
           attach method UpdateRecord to o::changeEvent
       
       end method. 
       
       method-id AddressToPropertyRecord(#address as type Address
                                   rentalRecord as type RentalPropertyFile+Property) private.
           set rentalRecord::Address1 to #address::Address1
           set rentalRecord::City to #address::City
           set rentalRecord::State to #address::State
           set rentalRecord::PostalCode to #address::PostalCode
       end method.
       
       method-id AddressToPersonRecord(#address as type Address        
                                       personRecord as type PersonFile+Person).
           if #address <> null                                
               set personRecord::Address1 to #address::Address1
               set personRecord::City to #address::City
               set personRecord::State to #address::State
               set personRecord::PostalCode to #address::PostalCode
           end-if
       end method.                                 
       
       method-id RecordToProperty(rentalRecord as type RentalPropertyFile+Property)
                       returning rentalProperty as type RentalProperty private.
           declare rentalAddress = new Address(rentalRecord::Address1 rentalRecord::City
                                               rentalRecord::State rentalRecord::PostalCode)
           set rentalProperty to new RentalProperty(rentalAddress)
           set rentalProperty::MonthlyRent to rentalRecord::Rent 
           
           if rentalRecord::LandlordEmail <> spaces and rentalRecord::LandlordEmail <> low-values
               if landlordDictionary contains rentalRecord::LandlordEmail::TrimSpaces()
                   declare landlord = landlordDictionary[rentalRecord::LandlordEmail::TrimSpaces()]
                   set rentalProperty::Owner = landlord
                   invoke landlord::AddProperty(rentalProperty)
               end-if
           end-if
       end method.  
       
       method-id RecordToPerson(personRecord as type PersonFile+Person)
                      returning person as type Person private. 
           evaluate  personRecord::Type 
           when "L"
               set person to new Landlord(personRecord::Name personRecord::Email)
           when "T"
               set person to new Tenant(personRecord::Name personRecord::Email)
           when other
               raise new PersistenceException ("Invalid type in person record: " & personRecord::Type)
           end-evaluate
           declare addressString = personRecord::Address::TrimSpaces()
           if size of addressString > 0 
               set person::Address to new Address(personRecord::Address1 personRecord::City
                                                  personRecord::State personRecord::PostalCode)
           end-if
       end method. 
       
       method-id PropertyToRecord(rentalProperty as type RentalProperty
                                  rentalRecord as type RentalPropertyFile+Property) private.
           invoke AddressToPropertyRecord(rentalProperty::Address rentalRecord)
           if rentalProperty::Owner <> null
               set rentalRecord::LandlordEmail to rentalProperty::Owner::Email
           end-if
           if rentalProperty::Lease <> null
               set rentalRecord::LeaseId to rentalProperty::Lease::Id
           end-if
           set rentalRecord::Rent to rentalProperty::MonthlyRent
       end method.          
       
       method-id PersonToRecord(person as type Person 
                                personRecord as type PersonFile+Person) private.
           invoke AddressToPersonRecord(person::Address personRecord)
           set personRecord::Email to person::Email
           set personRecord::Name to person::Name
           evaluate true
           when person instance of type Tenant
               set personRecord::Type to "T"
           when person instance of type Landlord
               set personRecord::Type to "L" 
           when other
      *>       We should never get here; if we do there is a coding error, or mismatch between
      *>       project library versions. 
               raise new Exception("Internal error, attempt to store unrecognised person type")
           end-evaluate
       end method.         
        
       method-id LeaseToRecord(lease as type Lease leaseRecord as type LeaseFile+Lease) private. 
           set leaseRecord::Id to lease::Id
           set leaseRecord::StartDay to lease::Start::Day
           set leaseRecord::StartMonth to lease::Start::Month
           set leaseRecord::StartYear to lease::Start::Year
           set leaseRecord::ExpiryDay to lease::End::Day
           set leaseRecord::ExpiryMonth to lease::End::Month
           set leaseRecord::ExpiryYear to lease::End::Year
           if lease::IsCancelled()
               set leaseRecord::Cancelled to "C"
           end-if
           if lease::Tenant <> null
               set leaseRecord::TenantEmail to lease::Tenant::Email
           end-if
           set leaseRecord::Address1 to lease::Property::Address::Address1
           set leaseRecord::City to lease::Property::Address::City
           set leaseRecord::State to lease::Property::Address::State
           set leaseRecord::PostalCode to lease::Property::Address::PostalCode
       end method.
       
       method-id RecordToLease(leaseRecord  as type LeaseFile+Lease) 
                     returning lease as type Lease private.
           declare startDate = type DateFactory::GetNewDate(leaseRecord::StartYear
                                                            leaseRecord::StartMonth
                                                            leaseRecord::StartDay)
           declare endDate = type DateFactory::GetNewDate(leaseRecord::ExpiryYear
                                                            leaseRecord::ExpiryMonth
                                                            leaseRecord::ExpiryDay)
           declare x = leaseRecord::TenantEmail
           declare t = tenantDictionary[leaseRecord::TenantEmail::TrimSpaces()] 
           declare leaseAddress = new Address(leaseRecord::Address1 leaseRecord::City
                                          leaseRecord::State leaseRecord::PostalCode)
           declare rental = rentalDictionary[leaseAddress]
           declare isCancelled = leaseRecord::Cancelled = "C"
           set lease to new Lease(startDate endDate rental t leaseRecord::Id isCancelled)
           
       end method.
       
       method-id UpdateRecord(changedRecord as type Updateable) private.
           evaluate true
           when changedRecord instance of type Person
               invoke self::Save(changedRecord as type Person)
           when changedRecord instance of type RentalProperty
               invoke self::Save(changedRecord as type RentalProperty)
           when changedRecord instance of type Lease
               invoke self::Save(changedRecord as type Lease)
           end-evaluate
       end method. 
       
       end class.
