////////////////////////////////////////////////////////////////////////////
///
/// Copyright(C) Micro Focus 2015-2016.
/// All rights reserved.
///
/// This sample code is supplied for demonstration purposes only
/// on an "as is" basis and is for use at your own risk.
///
////////////////////////////////////////////////////////////////////////////
package com.microfocus.javaleases;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import MicroFocus.AgencyText.Messages;
import MicroFocus.COBOL.Examples.Lettings.Landlord;
import MicroFocus.COBOL.Examples.Lettings.RentalProperty;
import MicroFocus.COBOL.Examples.Lettings.Updateable;
import MicroFocus.COBOL.Store.AgencyStore;
import MicroFocus.Examples.AgencyBridge.RecordValidator;

import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;

import com.microfocus.javaleases.components.PersonFields;

import javax.swing.JLabel;

import com.microfocus.javaleases.components.RecordList;

import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.JButton;

@SuppressWarnings("serial")
public class LandlordView extends JFrame
{
    private Landlord landlord ;
    private AgencyStore store; 
   
    private JPanel contentPane;
    private PersonFields personFields;
    private RecordList ownedList;

    /**
     * Create the frame.
     */
    public LandlordView(final MainWindow parent, AgencyStore store, Landlord landlord )
    {
        setTitle("Landlord view");
        setType(Type.UTILITY);
        this.store = store; 
        this.landlord = landlord;
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        setBounds(100, 100, 530, 335);
        contentPane = new JPanel();
        contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
        setContentPane(contentPane);
        
        personFields = new PersonFields();
        
        JLabel lblOwnedProperties = new JLabel("Owned");
        
        ownedList = new RecordList();
        
        JButton btnAdd = new JButton("Add");
        btnAdd.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                btnAddClick();
            }
        });
        
        JButton okBtn = new JButton("Done");
        okBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
                parent.closeMe(LandlordView.this);
            }
        });
        GroupLayout gl_contentPane = new GroupLayout(contentPane);
        gl_contentPane.setHorizontalGroup(
            gl_contentPane.createParallelGroup(Alignment.LEADING)
                .addGroup(gl_contentPane.createSequentialGroup()
                    .addGroup(gl_contentPane.createParallelGroup(Alignment.LEADING)
                        .addGroup(gl_contentPane.createSequentialGroup()
                            .addGap(20)
                            .addGroup(gl_contentPane.createParallelGroup(Alignment.LEADING)
                                .addComponent(personFields, GroupLayout.PREFERRED_SIZE, 340, GroupLayout.PREFERRED_SIZE)
                                .addGroup(gl_contentPane.createSequentialGroup()
                                    .addGroup(gl_contentPane.createParallelGroup(Alignment.LEADING)
                                        .addComponent(btnAdd)
                                        .addComponent(lblOwnedProperties))
                                    .addPreferredGap(ComponentPlacement.RELATED)
                                    .addComponent(ownedList, GroupLayout.PREFERRED_SIZE, 356, GroupLayout.PREFERRED_SIZE))))
                        .addGroup(gl_contentPane.createSequentialGroup()
                            .addGap(196)
                            .addComponent(okBtn)))
                    .addContainerGap(71, Short.MAX_VALUE))
        );
        gl_contentPane.setVerticalGroup(
            gl_contentPane.createParallelGroup(Alignment.LEADING)
                .addGroup(gl_contentPane.createSequentialGroup()
                    .addComponent(personFields, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
                    .addGroup(gl_contentPane.createParallelGroup(Alignment.LEADING)
                        .addGroup(gl_contentPane.createSequentialGroup()
                            .addGap(46)
                            .addComponent(lblOwnedProperties)
                            .addGap(38)
                            .addComponent(btnAdd))
                        .addGroup(gl_contentPane.createSequentialGroup()
                            .addGap(18)
                            .addComponent(ownedList, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)))
                    .addGap(18)
                    .addComponent(okBtn)
                    .addContainerGap(22, Short.MAX_VALUE))
        );
        contentPane.setLayout(gl_contentPane);
    }
    
    public void setVisible(boolean visible)
    {
        super.setVisible(visible);
        if (visible)
        {
            ownedList.clear();
            personFields.setPersonName(landlord.getName());
            personFields.setEmail(landlord.getEmail().getAddress());
            for (RentalProperty nextRental : landlord.GetProperties())
            {
                ownedList.addRecord(nextRental); 
            }
        }
    }
    
    public void btnAddClick()
    {
        final ItemSelector selector = new ItemSelector(this, 
                store.GetRentalProperties(), 
                new RecordValidator(){
                    public boolean Validate(Updateable record)
                    {
                        RentalProperty rp = (RentalProperty) record;
                        return (rp.getOwner() == null); 
                    }
                }); 
        selector.setTitle(Messages.PROPERTY_SELECTOR_TITLE);
        selector.addActionListener(new ActionListener(){
            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (e.getActionCommand().equalsIgnoreCase("OK"))
                {
                    RentalProperty rp = (RentalProperty) selector.getSelected() ; 
                    landlord.AddProperty(rp);
                    ownedList.addRecord(rp);
                }
                selector.setVisible(false);
                selector.dispose();
            }
        });
        selector.setVisible(true);
    }
}
