////////////////////////////////////////////////////////////////////////////
///
/// Copyright(C) Micro Focus 2015-2016.
/// All rights reserved.
///
/// This sample code is supplied for demonstration purposes only
/// on an "as is" basis and is for use at your own risk.
///
////////////////////////////////////////////////////////////////////////////
package com.microfocus.javaleases;

import java.awt.EventQueue;

import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JRadioButton;

import MicroFocus.AgencyText.Messages;
import MicroFocus.COBOL.Examples.Lettings.Landlord;
import MicroFocus.COBOL.Examples.Lettings.RentalProperty;
import MicroFocus.COBOL.Examples.Lettings.Tenant;
import MicroFocus.COBOL.Examples.Lettings.Lease;
import MicroFocus.COBOL.Examples.Lettings.Updateable;
import MicroFocus.COBOL.Store.AgencyStore;
import MicroFocus.Examples.AgencyBridge.EventBridge;
import MicroFocus.Examples.AgencyBridge.EventReceiver;
import MicroFocus.Examples.AgencyBridge.RecordValidator;

import com.microfocus.javaleases.components.PersonSelector;
import com.microfocus.javaleases.components.RecordList;





import javax.swing.JButton;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.SwingConstants;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

@SuppressWarnings("serial")
public class MainWindow extends JFrame 
                        implements EventReceiver
{
   private JPanel contentPane;
    private RecordList recordList;
    private AgencyStore store;
    
    private ViewState   viewState ; 
    private JButton btnNew;
    private JButton btnNewLease;
    private JButton btnDetails;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args)
	{
		EventQueue.invokeLater(new Runnable()
		{
			public void run()
			{
				try
				{
					MainWindow frame = new MainWindow();
					frame.setVisible(true);
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the frame.
	 */
	public MainWindow()
	{
	    store = new AgencyStore();
	    store.LoadCache();
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setBounds(100, 100, 751, 400);
		contentPane = new JPanel();
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(contentPane);
		this.setTitle(Messages.MAIN_WINDOW_TITLE); 
		
		JRadioButton rdbtnRentalProperties = new JRadioButton("Rental Properties");
		
		rdbtnRentalProperties.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent arg0) {
                viewState = ViewState.RENTALS; 
		        populateListBox();
                setBtnEnableState();
		    }
		});
		
        rdbtnRentalProperties.setSelected(true);
        viewState = ViewState.RENTALS;
        
		JRadioButton rdbtnLeases = new JRadioButton("Leases");
		rdbtnLeases.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
		        viewState = ViewState.LEASES;  
		        populateListBox();
                setBtnEnableState();
		    }
		});
		
		JRadioButton rdbtnTenants = new JRadioButton("Tenants");
		rdbtnTenants.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
		        viewState = ViewState.TENANTS; 
		        populateListBox();
                setBtnEnableState();
		    }
		});
		
		JRadioButton rdbtnLandlords = new JRadioButton("Landlords");
		rdbtnLandlords.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
		        viewState = ViewState.LANDLORDS;
		        populateListBox();
                setBtnEnableState();
		    }
		});
		
		ButtonGroup group = new ButtonGroup(); 
		group.add(rdbtnRentalProperties);
		group.add(rdbtnLeases);
		group.add(rdbtnTenants);
		group.add(rdbtnLandlords);
		
		recordList = new RecordList();
		recordList.addListActionListener(new ListSelectionListener() {

            @Override
            public void valueChanged(ListSelectionEvent arg0)
            {
                setBtnEnableState();
            }
		    
		});
		
		btnNew = new JButton("New");
		btnNew.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
		        newBtnClicked();
		    }
		});
		
		btnDetails = new JButton("Details");
		btnDetails.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent arg0) {
		        updateSelectedItemDetails();
		        setBtnEnableState();
		    }
		});
		
		btnNewLease = new JButton("New Lease");
		btnNewLease.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
	           if (viewState == ViewState.RENTALS) // safety check
	    	   {
	    	       NewLease newLease = new NewLease(MainWindow.this, store, (RentalProperty) 
	                                                    recordList.getSelectedItem());
	    	       newLease.setVisible(true);
	               }
			}
		});

		GroupLayout gl_contentPane = new GroupLayout(contentPane);
		gl_contentPane.setHorizontalGroup(
		    gl_contentPane.createParallelGroup(Alignment.LEADING)
		        .addGroup(gl_contentPane.createSequentialGroup()
		            .addContainerGap()
		            .addGroup(gl_contentPane.createParallelGroup(Alignment.LEADING)
		                .addGroup(gl_contentPane.createSequentialGroup()
		                    .addComponent(recordList, GroupLayout.PREFERRED_SIZE, 504, GroupLayout.PREFERRED_SIZE)
		                    .addPreferredGap(ComponentPlacement.RELATED, 72, Short.MAX_VALUE)
		                    .addGroup(gl_contentPane.createParallelGroup(Alignment.LEADING)
		                        .addComponent(btnNew)
		                        .addComponent(btnNewLease)
		                        .addComponent(btnDetails))
		                    .addGap(54))
		                .addGroup(gl_contentPane.createSequentialGroup()
		                    .addComponent(rdbtnRentalProperties)
		                    .addGap(18)
		                    .addComponent(rdbtnLeases)
		                    .addGap(29)
		                    .addComponent(rdbtnTenants)
		                    .addGap(49)
		                    .addComponent(rdbtnLandlords)
		                    .addContainerGap(315, Short.MAX_VALUE))))
		);
		gl_contentPane.setVerticalGroup(
		    gl_contentPane.createParallelGroup(Alignment.LEADING)
		        .addGroup(gl_contentPane.createSequentialGroup()
		            .addGroup(gl_contentPane.createParallelGroup(Alignment.LEADING)
		                .addGroup(gl_contentPane.createSequentialGroup()
		                    .addGap(21)
		                    .addGroup(gl_contentPane.createParallelGroup(Alignment.BASELINE)
		                        .addComponent(rdbtnRentalProperties)
		                        .addComponent(rdbtnLeases)
		                        .addComponent(rdbtnTenants)
		                        .addComponent(rdbtnLandlords))
		                    .addGap(18)
		                    .addComponent(recordList, GroupLayout.PREFERRED_SIZE, 248, GroupLayout.PREFERRED_SIZE))
		                .addGroup(gl_contentPane.createSequentialGroup()
		                    .addGap(74)
		                    .addComponent(btnNew)
		                    .addGap(43)
		                    .addComponent(btnDetails)
		                    .addGap(49)
		                    .addComponent(btnNewLease)))
		            .addContainerGap(42, Short.MAX_VALUE))
		);
		gl_contentPane.linkSize(SwingConstants.HORIZONTAL, new Component[] {btnNew, btnDetails, btnNewLease});
		contentPane.setLayout(gl_contentPane);
		EventBridge.RegisterForClassEvents(this); 
	}
	
	public void setVisible(boolean isVisible)
	{
	    super.setVisible(isVisible);
	    populateListBox();
        setBtnEnableState();
	}
	    
    private void updateSelectedItemDetails()
    {
        Updateable item = recordList.getSelectedItem();
        if (item != null)
        {
            switch (viewState)
            {
            case LANDLORDS:
                LandlordView landlordView = new LandlordView(this, store, (Landlord) item); 
                landlordView.setVisible(true);
                break;
            case TENANTS:
                TenantView tenantView = new TenantView(this, (Tenant) item); 
                tenantView.setVisible(true);
                break;
            case RENTALS:
                PropertyView propertyView = new PropertyView(this, store, (RentalProperty) item); 
                propertyView.setVisible(true);
                break;
            case LEASES:
                LeaseView leaseView = new LeaseView(this, (Lease) item); 
                leaseView.setVisible(true);
                break;
            }
        }
    }
    
    private void populateListBox()
	{
        recordList.clear(); 
	    switch (viewState)
	    {
	    case LANDLORDS:
	        populateLandlords();
	        break;
	    case TENANTS:
	        populateTenants();
	        break;
	    case RENTALS:
	        populateRentalProperties();
	        break;
	    case LEASES:
	        populateLeases();
            break;
	    }
	}
	
	private void populateRentalProperties()
	{
	    recordList.setListSource(store.GetRentalProperties(), new RecordValidator(){

            @Override
            public boolean Validate(Updateable nextRecord)
            {
                EventBridge.Register( MainWindow.this, nextRecord);
                return true; 
            }

	    });
	    recordList.populateListBox();
	}
	
	   private void populateTenants()
	    {
	        recordList.setListSource(store.GetTenants(), new RecordValidator(){

	            @Override
	            public boolean Validate(Updateable nextRecord)
	            {
	                EventBridge.Register( MainWindow.this, nextRecord);
	                return true; 
	            }

	        });
	        recordList.populateListBox();
	    }
	   
	    private void populateLandlords()
	    {
	        recordList.setListSource(store.GetLandlords(), new RecordValidator(){

	            @Override
	            public boolean Validate(Updateable nextRecord)
	            {
	                EventBridge.Register( MainWindow.this, nextRecord);
	                return true; 
	            }

	        });
	        recordList.populateListBox();
	    }
	    
	    private void populateLeases()
	    {
	        recordList.setListSource(store.GetLeases(), new RecordValidator(){

	            @Override
	            public boolean Validate(Updateable nextRecord)
	            {
	                EventBridge.Register( MainWindow.this, nextRecord);
	                return true; 
	            }

	        });
	        recordList.populateListBox();
	    }
	    
    private boolean shouldUpdateList(Object o) 
    {
        switch (viewState)
        {
        case LANDLORDS:
            if (o instanceof Landlord)
                return true;
        case TENANTS:
            if (o instanceof Tenant)
                return true;
        case RENTALS:
            if (o instanceof RentalProperty)
                return true; 
        case LEASES:
            if (o instanceof Lease)
                return true;
        }
        return  false;
    }
    
    private void newBtnClicked()
    {
        switch (viewState)
        {
        case LANDLORDS:
            NewPerson landlordDlg = new NewPerson(this, store, PersonSelector.LANDLORD);
            landlordDlg.setVisible(true);
            break;
        case TENANTS:
            NewPerson tenantDlg = new NewPerson(this, store, PersonSelector.LANDLORD);
            tenantDlg.setVisible(true);
            break;
        case RENTALS: 
            PropertyView pv = new PropertyView(this, store);
            pv.setVisible(true);
            break; 
        case LEASES:
            break;
        }
    }
    
    

    /**
     * Event handler fired when a new record is created. 
     */
	@Override
    public void NewRecord(Updateable record)
    {
	    EventBridge.Register( MainWindow.this, record);
        if (shouldUpdateList(record))
        {
            recordList.addRecord(record);
        }
    }
    
    private void setBtnEnableState()
    {
	    if (viewState == ViewState.RENTALS && recordList.getSelectedItem() != null)
	    {
	        RentalProperty rental = (RentalProperty) recordList.getSelectedItem();
	        btnNewLease.setEnabled(rental.getLease() == null);
	    }
	    else
	    {
	        btnNewLease.setEnabled(false);
	    }
        btnNew.setEnabled(viewState != ViewState.LEASES);
    }

    
    /**
     * Close a child window. 
     * @param child
     */
    void closeMe(JFrame child)
    {
        child.setVisible(false);
        child.dispose();
    }
    
     @Override
    public void UpdatedRecord(Updateable record)
    {
        if (shouldUpdateList(record))
        {
            recordList.updateRecord(record);
        }
        
    }
    
    enum ViewState
    {
        RENTALS,
        TENANTS,
        LANDLORDS,
        LEASES
    }
}
