////////////////////////////////////////////////////////////////////////////
///
/// Copyright(C) Micro Focus 2015-2016.
/// All rights reserved.
///
/// This sample code is supplied for demonstration purposes only
/// on an "as is" basis and is for use at your own risk.
///
////////////////////////////////////////////////////////////////////////////
package com.microfocus.javaleases;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.SwingConstants;

import java.awt.Component;

import MicroFocus.AgencyText.Messages;
import MicroFocus.COBOL.Examples.Lettings.Landlord;
import MicroFocus.COBOL.Examples.Lettings.Tenant;
import MicroFocus.COBOL.Examples.Support.Email;
import MicroFocus.COBOL.Store.AgencyStore;

import com.microfocus.javaleases.components.PersonFields;
import com.microfocus.javaleases.components.PersonSelector;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

@SuppressWarnings("serial")
public class NewPerson extends JFrame
{
    private JPanel contentPane;
    private PersonSelector selector; 
    private AgencyStore store; 
    private PersonFields personFields;
    private MainWindow parent; 

    /**
     * Create the frame.
     */
    public NewPerson(MainWindow parent, AgencyStore store, PersonSelector selector)
    {
        this.store = store; 
        this.selector = selector; 
        this.parent = parent; 
        switch (selector)
        {
        case LANDLORD:
            setTitle(Messages.NEW_LANDLORD_TITLE);
            break;
        case TENANT:
            setTitle(Messages.NEW_TENANT_TITLE);
            break;
        }
        setType(Type.UTILITY);
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        setBounds(100, 100, 363, 177);
        contentPane = new JPanel();
        contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
        setContentPane(contentPane);
        
        JButton btnOk = new JButton("Ok");
        btnOk.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                btnOkClick(); 
            }
        });
        
        JButton btnCancel = new JButton("Cancel");
        btnCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                btnCancelClick();
            }
        });
        
        personFields = new PersonFields();
        GroupLayout gl_contentPane = new GroupLayout(contentPane);
        gl_contentPane.setHorizontalGroup(
            gl_contentPane.createParallelGroup(Alignment.LEADING)
                .addGroup(gl_contentPane.createSequentialGroup()
                    .addGroup(gl_contentPane.createParallelGroup(Alignment.LEADING)
                        .addGroup(gl_contentPane.createSequentialGroup()
                            .addGap(92)
                            .addComponent(btnOk)
                            .addGap(50)
                            .addComponent(btnCancel))
                        .addGroup(gl_contentPane.createSequentialGroup()
                            .addContainerGap()
                            .addComponent(personFields, GroupLayout.PREFERRED_SIZE, 317, GroupLayout.PREFERRED_SIZE)))
                    .addContainerGap(133, Short.MAX_VALUE))
        );
        gl_contentPane.setVerticalGroup(
            gl_contentPane.createParallelGroup(Alignment.LEADING)
                .addGroup(gl_contentPane.createSequentialGroup()
                    .addComponent(personFields, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
                    .addPreferredGap(ComponentPlacement.RELATED)
                    .addGroup(gl_contentPane.createParallelGroup(Alignment.BASELINE)
                        .addComponent(btnOk)
                        .addComponent(btnCancel))
                    .addContainerGap(GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        gl_contentPane.linkSize(SwingConstants.HORIZONTAL, new Component[] {btnOk, btnCancel});
        contentPane.setLayout(gl_contentPane);
    }
    
    private void btnOkClick()
    {
        if (personFields.getEmail().length() == 0 || personFields.getPersonName().length() == 0 )
        {
            JOptionPane.showMessageDialog(this,
                    Messages.NEW_PERSON_EMPTY_FIELDS,
                    Messages.ADD_PERSON_MSG_TITLE,
                    JOptionPane.ERROR_MESSAGE);
        }
        else
        {
            if (Email.Validate(personFields.getEmail()))
            {
                switch (selector)
                {
                case LANDLORD:
                    Landlord l = new Landlord(personFields.getPersonName(), personFields.getEmail());
                    store.Save(l); 
                    break;
                case TENANT:
                    Tenant t = new Tenant(personFields.getPersonName(), personFields.getEmail());
                    store.Save(t);
                    break;
                }
                parent.closeMe(this);
            }
            else
            {
                JOptionPane.showMessageDialog(this,
                        Messages.INVALID_EMAIL_ADDRESS,
                        Messages.ADD_PERSON_MSG_TITLE,
                        JOptionPane.ERROR_MESSAGE);
            }
        }
    }
    
    private void btnCancelClick()
    {
        parent.closeMe(this);
    }
}
