////////////////////////////////////////////////////////////////////////////
///
/// Copyright(C) Micro Focus 2015-2016.
/// All rights reserved.
///
/// This sample code is supplied for demonstration purposes only
/// on an "as is" basis and is for use at your own risk.
///
////////////////////////////////////////////////////////////////////////////
package com.microfocus.javaleases;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;

import MicroFocus.AgencyText.Messages;
import MicroFocus.COBOL.Examples.Lettings.RentalProperty;
import MicroFocus.COBOL.Examples.Lettings.Updateable;
import MicroFocus.COBOL.Examples.Support.Address;
import MicroFocus.COBOL.Store.AgencyStore;
import MicroFocus.COBOL.Store.PersistenceException;
import MicroFocus.Examples.AgencyBridge.EventReceiver;
import MicroFocus.Examples.AgencyBridge.EventBridge;

import com.microfocus.javaleases.components.AddressFields;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.JButton;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.SwingConstants;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

@SuppressWarnings("serial")
public class PropertyView extends JFrame implements EventReceiver
{

    private JPanel contentPane;
    private JTextField rentField;
    private AgencyStore store; 
    private RentalProperty rp; 
    private AddressFields addressFields;
    
    private JButton btnOk ; 
    private MainWindow parent; 
    private JLabel lblMonthlyRent; 
    private JCheckBox checkboxLeased; 

    public PropertyView(MainWindow parent, AgencyStore store)
    {
        this();
        this.store = store;
        this.parent = parent; 
    }
    
    public PropertyView(MainWindow parent, AgencyStore store, RentalProperty rp)
    {
        this(parent, store);
        this.rp = rp;
        EventBridge.Register(this, rp); 
    }
    
    /**
     * Create the frame.
     */
    public PropertyView()
    {
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        setBounds(100, 100, 515, 360);
        contentPane = new JPanel();
        contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
        setContentPane(contentPane);
        setTitle(Messages.RENTAL_VIEW_TITLE);

        addressFields = new AddressFields();
        
        lblMonthlyRent = new JLabel("Monthly Rent");
        
        rentField = new JTextField();
        rentField.setColumns(10);
        
        checkboxLeased = new JCheckBox("Leased");
        checkboxLeased.setEnabled(false);
        
        btnOk = new JButton("OK");
        btnOk.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                okBtn_Click();
            }
        });
        
        JButton btnCancel = new JButton("Cancel");
        btnCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                close();
            }
        });
        
        JButton btnApply = new JButton("Apply");
        btnApply.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                UpdateRent();
            }
        });
        GroupLayout gl_contentPane = new GroupLayout(contentPane);
        gl_contentPane.setHorizontalGroup(
            gl_contentPane.createParallelGroup(Alignment.LEADING)
                .addGroup(gl_contentPane.createSequentialGroup()
                    .addGroup(gl_contentPane.createParallelGroup(Alignment.LEADING)
                        .addGroup(gl_contentPane.createSequentialGroup()
                            .addContainerGap()
                            .addComponent(addressFields, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addGroup(gl_contentPane.createSequentialGroup()
                            .addGap(48)
                            .addComponent(lblMonthlyRent)
                            .addGap(44)
                            .addGroup(gl_contentPane.createParallelGroup(Alignment.LEADING)
                                .addGroup(gl_contentPane.createSequentialGroup()
                                    .addComponent(btnOk)
                                    .addPreferredGap(ComponentPlacement.RELATED, 64, Short.MAX_VALUE)
                                    .addComponent(btnApply)
                                    .addGap(51)
                                    .addComponent(btnCancel))
                                .addGroup(gl_contentPane.createSequentialGroup()
                                    .addComponent(rentField, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
                                    .addGap(66)
                                    .addComponent(checkboxLeased)))))
                    .addGap(29))
        );
        gl_contentPane.setVerticalGroup(
            gl_contentPane.createParallelGroup(Alignment.TRAILING)
                .addGroup(gl_contentPane.createSequentialGroup()
                    .addContainerGap()
                    .addComponent(addressFields, GroupLayout.DEFAULT_SIZE, 206, Short.MAX_VALUE)
                    .addGap(18)
                    .addGroup(gl_contentPane.createParallelGroup(Alignment.BASELINE)
                        .addComponent(lblMonthlyRent)
                        .addComponent(rentField, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
                        .addComponent(checkboxLeased))
                    .addGap(18)
                    .addGroup(gl_contentPane.createParallelGroup(Alignment.BASELINE)
                        .addComponent(btnOk)
                        .addComponent(btnCancel)
                        .addComponent(btnApply))
                    .addGap(35))
        );
        gl_contentPane.linkSize(SwingConstants.HORIZONTAL, new Component[] {btnOk, btnCancel});
        contentPane.setLayout(gl_contentPane);
    }
    
    
    /**
     * If we are displaying an existing rental property, 
     * set the data into the view fields and disable them (apart from 
     * the rent field). 
     * Otherwise enable them for editing. 
     */
    private void setupView()
    {
        if (rp == null)
        {
            addressFields.setEnabled(true); 
        }
        else
        {
            addressFields.setAddress(rp.getAddress());
            addressFields.setEnabled(false);
            rentField.setText(Integer.toString(rp.getMonthlyRent()));
            checkboxLeased.setSelected(rp.getLease() != null);
        }
    }
    
    private void okBtn_Click()
    {
        if (rp == null)
        {
            rp = createRentalProperty();  
            if (rp != null)
            {
                try
                {
                    store.Save(rp);
                    close();
                }
                catch (Exception pe )
                {
                    if (pe instanceof PersistenceException )
                    {
                        JOptionPane.showMessageDialog(this,
                                Messages.PERSISTENCE_ERROR, 
                                Messages.ADD_RENTAL_MSG_TITLE,
                                JOptionPane.ERROR_MESSAGE); 
                    }
                    else
                        throw(pe) ;
                }
            }
        }
        else
        {
            UpdateRent();
            close();
        }
    }
    
    private void UpdateRent()
    {
        try
        {
            int rent = Integer.parseInt(rentField.getText());
            if (rent != rp.getMonthlyRent())
            {
                rp.setMonthlyRent(rent);;
                store.Save(rp);
            }
        }
        catch (NumberFormatException e)
        {
            displayNonNumericRentMessage();
        }        
    }
    
    /**
     * Close this frame. 
     */
    private void close()
    {
        if (rp != null)
        {
            EventBridge.Unregister(this, rp);
        }
        parent.closeMe(this);
        
    }

    private RentalProperty createRentalProperty()
    {
        if (addressFields.isAddressValid())
        {
            Address addressKey = addressFields.getAddress(); 
            RentalProperty existing = store.FindProperty(addressKey);
            if (existing == null)
            {
                try
                {
                    String rentText = rentField.getText();
                    int rent = Integer.parseInt(rentText);
                    RentalProperty rental = new RentalProperty(addressKey);
                    rental.setMonthlyRent(rent);
                    return rental;
                }
                catch (NumberFormatException nfe)
                {
                    displayNonNumericRentMessage();
                }
            }
            else
            {
                JOptionPane.showMessageDialog(this,
                        Messages.RENTAL_PROPERTY_EXISTS, 
                        Messages.ADD_RENTAL_MSG_TITLE,
                        JOptionPane.ERROR_MESSAGE); 
            }
                
        }
        else
        {
            JOptionPane.showMessageDialog(this,
                    Messages.ADDRESS_KEY_FIELDS_EMPTY, 
                    Messages.ADD_RENTAL_MSG_TITLE,
                    JOptionPane.ERROR_MESSAGE); 
        }
        return null;
    }
    
    /**
     * Pop up a message box for a non-numeric value in the rent field. 
     */
    private void displayNonNumericRentMessage()
    {
        JOptionPane.showMessageDialog(this,
                Messages.NUMERIC_RENTAL_VALUE, 
                Messages.ADD_RENTAL_MSG_TITLE,
                JOptionPane.ERROR_MESSAGE); 
    }
    
    public void setVisible(boolean visible)
    {
        super.setVisible(visible);
        setupView(); 
    }
    
    public void NewRecord(Updateable o ) 
    {
        // Do nothing for this event
    }
    
    public void UpdatedRecord(Updateable o )
    {
        if (o instanceof RentalProperty)
        {
            rentField.setText(Integer.toString(((RentalProperty)o).getMonthlyRent()).toString());
        }
    }
}
