////////////////////////////////////////////////////////////////////////////
///
/// Copyright(C) Micro Focus 2015-2016.
/// All rights reserved.
///
/// This sample code is supplied for demonstration purposes only
/// on an "as is" basis and is for use at your own risk.
///
////////////////////////////////////////////////////////////////////////////
package com.microfocus.javaleases.components;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import javax.swing.InputVerifier;
import javax.swing.JComponent;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import MicroFocus.COBOL.Examples.Support.DateJvm;
import MicroFocus.COBOL.Examples.Support.IDate;

@SuppressWarnings("serial")
public class DateField extends JTextField
{
    private SimpleDateFormat dateFormatter; 
    private String datePattern; 
    private ActionListener listener; 
    private boolean textChangeFlag; 
    
    public DateField()
    {
        super();
        // Get the simple format for this locale and remove the time portion. 
        String[] parts  = new SimpleDateFormat().toPattern().split(" ");
        datePattern = parts.length > 0 ? parts[0] : "dd/MM/yy"; 
        dateFormatter = new SimpleDateFormat(datePattern); 
        dateFormatter.setLenient(false);        
        
        setColumns(10);
        setText(dateFormatter.toPattern());
        setInputVerifier(new InputVerifier(){

            @Override
            public boolean verify(JComponent input)
            {
                if (isValidDate())
                {
                    if (listener != null)
                    {
                        ActionEvent ae = new ActionEvent(DateField.this, ActionEvent.ACTION_FIRST, "DATE");
                        listener.actionPerformed(ae);
                    }
                    return true;
                }
                return !textChangeFlag; // valid 
            }
        });
        
        getDocument().addDocumentListener(new DocumentListener(){

            @Override
            public void changedUpdate(DocumentEvent arg0){}

            @Override
            public void insertUpdate(DocumentEvent arg0)
            {
                raiseValidated();
            }

            @Override
            public void removeUpdate(DocumentEvent arg0)
            {
                raiseValidated();
            }
            
            public void raiseValidated()
            {
                textChangeFlag = true; 
                boolean valid = isValidDate(); 
                if (listener != null)
                {
                    String actionState = valid ? "VALID" : "INVALID";
                    ActionEvent ae = new ActionEvent(DateField.this, ActionEvent.ACTION_FIRST, actionState);
                    listener.actionPerformed(ae);
                }                    
            }
        });
        
        addFocusListener(new FocusListener(){

            @Override
            public void focusGained(FocusEvent arg0)
            {
                DateField.this.setSelectionStart(0);
                DateField.this.setSelectionEnd(DateField.this.getText().length());
            }
            @Override
            public void focusLost(FocusEvent arg0) {}
        });
    }
    
    public IDate getDate()
    {
        String text = getText(); 
        if (text != null && text.length() > 0)
        {
            try
            {
               Date d = dateFormatter.parse(text); 
               Calendar c = Calendar.getInstance();
               c.setTime(d);
               return new DateJvm(c);        
            }
            catch(ParseException e)
            {
               // Do nothing
            }
        }
        return null;
    }
    
    public void addValidationListener(ActionListener listener)
    {
        this.listener = listener;
    }
    
    public void setDate(IDate date)
    {
        setText(date.toString());
    }
    
    /**
     * Helper method for InputVerifier on date fields. 
     * @param text
     * @return
     */
    public boolean isValidDate()
    {
        try
        {
            dateFormatter.parse(getText());
            return true;
        }
        catch (ParseException e)
        {
            return false;
        }
    }
}
