////////////////////////////////////////////////////////////////////////////
///
/// Copyright(C) Micro Focus 2015-2016.
/// All rights reserved.
///
/// This sample code is supplied for demonstration purposes only
/// on an "as is" basis and is for use at your own risk.
///
////////////////////////////////////////////////////////////////////////////
package com.microfocus.javaleases.components;

import javax.swing.JPanel;

import java.awt.BorderLayout;

import javax.swing.DefaultListModel;
import javax.swing.JScrollPane;
import javax.swing.JList;
import javax.swing.event.ListSelectionListener;

import MicroFocus.COBOL.Examples.Lettings.Updateable;
import MicroFocus.Examples.AgencyBridge.RecordValidator;

public class RecordList extends JPanel
{
    private JList<Updateable> list; 
    private DefaultListModel<Updateable>        model; 
    private Iterable<? extends Updateable>      iter; 
    private RecordValidator                     validator; 


    /**
     * 
     */
    private static final long serialVersionUID = 3170831074963849826L;

    /**
     * Create the panel.
     */
    public RecordList()
    {
        setLayout(new BorderLayout(0, 0));
        model = new DefaultListModel<Updateable>();
        
        JScrollPane scrollPane = new JScrollPane();
        add(scrollPane, BorderLayout.CENTER);
        
        list = new JList<Updateable>(model);
        scrollPane.setViewportView(list);

    }
    
    
    public void setListSource(Iterable<? extends Updateable>iter, RecordValidator validator)
    {
        this.iter = iter;
        this.validator = validator; 
    }
    
    /**
     * Refresh an existing element which has changed. 
     * @param record
     */
    public void updateRecord(Updateable record)
    {
        int i = model.indexOf(record); 
        if (i > -1)
        {
            model.removeElementAt(i);
            model.insertElementAt(record, i);
        }
    }
    
    
    public void populateListBox()
    {
        for (Updateable next : iter)
        {
            if (validator.Validate(next))
            {
                model.addElement(next);
            }
        }
    }
    
    /**
     * Add a new record to the list. 
     * @param record
     */
    public void addRecord(Updateable record)
    {
        model.addElement(record);
    }
    
    public void clear()
    {
        model.clear();
    }
    
    public void addListActionListener(ListSelectionListener listener)
    {
        list.addListSelectionListener(listener);
    }
    
    /**
     * Return the record currently selected in the list box. 
     * @return
     */
    public Updateable getSelectedItem()
    {
        return list.getSelectedValue(); 
    }
    
    public void setVisible(boolean isVisible)
    {
        super.setVisible(isVisible);
        if (isVisible)
        {
            populateListBox();
        }
    }

}
