      **************************************************************************
      *
      * Copyright (C) Micro Focus 2015-2016.
      * All rights reserved.
      *
      * This sample code is supplied for demonstration purposes only
      * on an "as is" basis and is for use at your own risk.
      *
      **************************************************************************
       class-id WindowsLeases.MainWindow is partial
                 inherits type System.Windows.Forms.Form.
       
       working-storage section.
       01 store            type AgencyStore. 
       method-id NEW (store as type AgencyStore).
       procedure division.
           set self::store to store
           invoke self::InitializeComponent
           attach method NewRentalProperty to type RentalProperty::NewObjectEvent
           attach method NewLandlord to type Landlord::NewObjectEvent
           attach method NewTenant to type Tenant::NewObjectEvent
           attach method NewLease to type Lease::NewObjectEvent
           goback.
       end method.

       method-id OnShown(e as type EventArgs) override.
           invoke super::OnShown(e)
           invoke PopulateListBox()
       end method.

       method-id UpdateSelectedItemDetails() protected.
           declare item = self::displayList::SelectedItem  
           if item <> null
               evaluate true
               when btnRentals::Checked
                   declare rv = new PropertyView(store, item as type RentalProperty)
                   invoke rv::Show()
               when btnTenants::Checked
                   declare tv = new TenantView(item as type Tenant)
                   invoke tv::Show()
               when btnLeases::Checked
                   declare lv = new LeaseView(item as type Lease)
                   invoke lv::Show()
               when btnLandlords::Checked
                   declare lv = new LandlordView(store, item as type Landlord)
                   invoke lv::Show()
               end-evaluate
           end-if
       end method. 
       
       method-id PopulateListBox() private.
           invoke displayList::BeginUpdate() *> Stop painting the control while we update
           invoke displayList::Items::Clear()
           evaluate  true
           when btnRentals::Checked
               invoke PopulateRentalProperties()
           when btnTenants::Checked
               invoke PopulateTenants()
           when btnLandlords::Checked
               invoke PopulateLandlords()
           when btnLeases::Checked
               invoke PopulateLeases()
           end-evaluate
           invoke displayList::EndUpdate()
           invoke setBtnEnableState()
       end method. 
       
       method-id PopulateRentalProperties() private. 
           perform varying nextRental as type RentalProperty through store::GetRentalProperties()
               invoke self::displayList::Items::Add(nextRental)
               invoke self::RegisterHandler(nextRental) 
           end-perform
       end method. 

       method-id PopulateTenants() private. 
           perform varying nextTenant as type Tenant through store::GetTenants()
               invoke self::displayList::Items::Add(nextTenant)
               invoke self::RegisterHandler(nextTenant) 
           end-perform
       end method. 
       
       method-id PopulateLandlords() private. 
           perform varying nextLandlord as type Landlord through store::GetLandlords()
               invoke self::displayList::Items::Add(nextLandlord)
               invoke self::RegisterHandler(nextLandlord) 
           end-perform
       end method.        

       method-id PopulateLeases() private. 
           perform varying nextLease as type Lease through store::GetLeases()
               invoke self::displayList::Items::Add(nextLease)
               invoke self::RegisterHandler(nextLease) 
           end-perform
       end method.        
       
       method-id ShouldUpdateList(o as object) returning updateList as condition-value private. 
           evaluate true
           when btnRentals::Checked and o instance of type RentalProperty
               set updateList to true
           when btnTenants::Checked and o instance of type Tenant
               set updateList to true
           when btnLandlords::Checked and o instance of type Landlord
               set updateList to true
           when btnLeases::Checked and o instance of type Lease
               set updateList to true
           end-evaluate
       end method. 

       method-id RegisterHandler(item as type Updateable).
           detach method ItemUpdated from item::changeEvent
           attach method ItemUpdated to item::changeEvent
       end method. 
       
       method-id ItemUpdated(item as type Updateable) private.
           if ShouldUpdateList(item)
               declare i = displayList::Items::IndexOf(item)
               invoke displayList::Items::Remove(item)
               invoke displayList::Items::Insert(i, item)
           end-if           
       end method. 
       
       method-id NewRentalProperty(o as type Updateable).
           if ShouldUpdateList(o)
               invoke displayList::Items::Add(o)
               attach method ItemUpdated to o::changeEvent
           end-if
       end method.
       
       method-id NewLandlord(o as type Updateable).
           if ShouldUpdateList(o)
               invoke displayList::Items::Add(o)
               attach method ItemUpdated to o::changeEvent
           end-if
       end method. 
       
       method-id NewTenant(o as type Updateable).
           if ShouldUpdateList(o)
               invoke displayList::Items::Add(o)
               attach method ItemUpdated to o::changeEvent
           end-if
       end method. 
       
       method-id NewLease(o as type Updateable).
      *> Different logic for NewLease() - these are created from the RentalProperty view
      *> but we want to switch to Lease view when created. 
           invoke btnLeases::Select()
       end method. 
       

       method-id displayList_DoubleClick final private.
       procedure division using by value sender as object e as type System.EventArgs.
           invoke self::UpdateSelectedItemDetails()
           invoke self::setBtnEnableState()
       end method.

       method-id detailsBtn_Click(sender as object e as type EventArgs) final private.
           invoke self::UpdateSelectedItemDetails()
           invoke setBtnEnableState()
       end method.

       method-id newBtn_Click final private.
       procedure division using by value sender as object e as type System.EventArgs.
           evaluate true
           when btnRentals::Checked
               declare rentalView = new PropertyView(store)
               invoke rentalView::Show()
           when btnTenants::Checked
               declare tenantView = new NewPerson(store, type PersonSelector::Tenant)
               invoke tenantView::Show()
           when btnLandlords::Checked
               declare landlordView = new NewPerson(store, type PersonSelector::Landlord)
               invoke landlordView::Show()
           end-evaluate
       end method.

       method-id btnRentals_CheckedChanged final private.
       procedure division using by value sender as object e as type System.EventArgs.
           invoke PopulateListBox()
       end method.

       method-id btnLeases_CheckedChanged final private.
       procedure division using by value sender as object e as type System.EventArgs.
           invoke PopulateListBox()
       end method.
       
       method-id btnTenants_CheckedChanged final private.
       procedure division using by value sender as object e as type System.EventArgs.
           invoke PopulateListBox()
       end method.

       method-id btnLandlords_CheckedChanged final private.
       procedure division using by value sender as object e as type System.EventArgs.
           invoke PopulateListBox()
       end method.
       
       method-id setBtnEnableState private.
           if btnRentals::Checked and displayList::SelectedItem <> null
               declare rental = displayList::SelectedItem as type RentalProperty
               set btnNewLease::Enabled to rental::Lease = null
           else
               set btnNewLease::Enabled to false
           end-if
           set newBtn::Enabled to not btnLeases::Checked
           goback.
       end method.

       method-id displayList_Click(sender as object, e as type System.EventArgs) final private.
           invoke setBtnEnableState()
       end method.

       method-id btnNewLease_Click(sender as object, e as type System.EventArgs) final private.
           declare rental = displayList::SelectedItem as type RentalProperty
           declare newLease = new NewLease(store, rental)
           invoke newLease::Show()
       end method.

       method-id button1_Click final private.
       procedure division using by value sender as object e as type System.EventArgs.
           declare propertyEnvvar = type Environment::GetEnvironmentVariable("dd_propertyfile")
           declare leaseEnvvar = type Environment::GetEnvironmentVariable("dd_leasefile")
           declare personEnvvar = type Environment::GetEnvironmentVariable("dd_personfile")
           declare currentFolder = type Directory::GetCurrentDirectory()
           declare propertyPath = type Path::Combine(currentFolder, propertyEnvvar) 
           declare leaseEPath = type Path::Combine(currentFolder, leaseEnvvar)
           declare personPath = type Path::Combine(currentFolder, personEnvvar)
           declare msg = type String::Format("Environment variables:\nProperty - {0}\nLease   - {1}\nPerson  - {2}\nFiles:\n{3}\n{4}\n{5}", 
                                             propertyEnvvar, leaseEnvvar, personEnvvar, propertyPath, leaseEPath, personPath)
          invoke type MessageBox::Show(self, 
                   msg,
                   "File storage",
                   type MessageBoxButtons::OK 
                   type MessageBoxIcon::Information)
       end method.

       end class.
